@@ -1,13 +1,15 @@
 use std::borrow::Cow;
 
 use duper::{
-    DuperArray, DuperIdentifier, DuperInner, DuperKey, DuperObject, DuperString, DuperValue,
+    DuperArray, DuperIdentifier, DuperInner, DuperKey, DuperObject, DuperString, DuperTemporal,
+    DuperValue,
 };
 use pyo3::{exceptions::PyValueError, prelude::*, types::*};
 
 use crate::{
     Duper,
     ser::{serialize_pyany, serialize_pyclass_identifier},
+    temporal::TemporalString,
 };
 
 #[derive(Debug)]
@@ -89,6 +91,16 @@ pub(crate) enum WellKnownType<'py> {
     Ulid(Bound<'py, PyAny>),
     // re
     Pattern(Bound<'py, PyAny>),
+    // temporal string
+    TemporalStringInstant(duper::DuperTemporal<'static>),
+    TemporalStringZonedDateTime(duper::DuperTemporal<'static>),
+    TemporalStringPlainDate(duper::DuperTemporal<'static>),
+    TemporalStringPlainTime(duper::DuperTemporal<'static>),
+    TemporalStringPlainDateTime(duper::DuperTemporal<'static>),
+    TemporalStringPlainYearMonth(duper::DuperTemporal<'static>),
+    TemporalStringPlainMonthDay(duper::DuperTemporal<'static>),
+    TemporalStringDuration(duper::DuperTemporal<'static>),
+    TemporalString(duper::DuperTemporal<'static>),
     // uuid
     Uuid(Bound<'py, PyAny>),
 }
@@ -310,6 +322,53 @@ impl<'py> WellKnownType<'py> {
         {
             return Ok(Some(WellKnownType::Dataclass(value.clone())));
         }
+        // temporal string
+        if let Ok(value) = value.cast::<TemporalString>() {
+            let temporal = &value.get().temporal;
+            match temporal {
+                DuperTemporal::Instant(_) => {
+                    return Ok(Some(WellKnownType::TemporalStringInstant(temporal.clone())));
+                }
+                DuperTemporal::ZonedDateTime(_) => {
+                    return Ok(Some(WellKnownType::TemporalStringZonedDateTime(
+                        temporal.clone(),
+                    )));
+                }
+                DuperTemporal::PlainDate(_) => {
+                    return Ok(Some(WellKnownType::TemporalStringPlainDate(
+                        temporal.clone(),
+                    )));
+                }
+                DuperTemporal::PlainTime(_) => {
+                    return Ok(Some(WellKnownType::TemporalStringPlainTime(
+                        temporal.clone(),
+                    )));
+                }
+                DuperTemporal::PlainDateTime(_) => {
+                    return Ok(Some(WellKnownType::TemporalStringPlainDateTime(
+                        temporal.clone(),
+                    )));
+                }
+                DuperTemporal::PlainYearMonth(_) => {
+                    return Ok(Some(WellKnownType::TemporalStringPlainYearMonth(
+                        temporal.clone(),
+                    )));
+                }
+                DuperTemporal::PlainMonthDay(_) => {
+                    return Ok(Some(WellKnownType::TemporalStringPlainMonthDay(
+                        temporal.clone(),
+                    )));
+                }
+                DuperTemporal::Duration(_) => {
+                    return Ok(Some(WellKnownType::TemporalStringDuration(
+                        temporal.clone(),
+                    )));
+                }
+                DuperTemporal::Unspecified(_) => {
+                    return Ok(Some(WellKnownType::TemporalString(temporal.clone())));
+                }
+            }
+        }
         Ok(None)
     }
 
@@ -340,36 +399,94 @@ impl<'py> WellKnownType<'py> {
                 inner: serialize_pyany(value.getattr("__dict__")?)?.inner,
             }),
             // datetime
-            WellKnownType::DateTime(value) => Ok(DuperValue {
-                identifier: Some(
-                    DuperIdentifier::try_from(Cow::Borrowed("DateTime")).expect("valid identifier"),
-                ),
-                inner: DuperInner::String(DuperString::from(Cow::Owned(
-                    value.call_method0("isoformat")?.extract()?,
-                ))),
+            WellKnownType::DateTime(value) => Ok({
+                let datetime: Cow<'_, str> =
+                    Cow::Owned(value.call_method0("isoformat")?.extract()?);
+                let tzinfo = value.getattr("tzinfo")?;
+                if tzinfo.is_none() || tzinfo.getattr("utcoffset")?.call1((value,))?.is_none() {
+                    DuperValue {
+                        identifier: Some(
+                            DuperIdentifier::try_from(Cow::Borrowed("PlainDateTime"))
+                                .expect("valid identifier"),
+                        ),
+                        inner: DuperInner::Temporal(
+                            DuperTemporal::try_plain_date_time_from(datetime).map_err(|err| {
+                                PyValueError::new_err(format!(
+                                    "Failed to parse Temporal value: {err}"
+                                ))
+                            })?,
+                        ),
+                    }
+                } else {
+                    DuperValue {
+                        identifier: Some(
+                            DuperIdentifier::try_from(Cow::Borrowed("Instant"))
+                                .expect("valid identifier"),
+                        ),
+                        inner: DuperInner::Temporal(
+                            DuperTemporal::try_instant_from(datetime).map_err(|err| {
+                                PyValueError::new_err(format!(
+                                    "Failed to parse Temporal value: {err}"
+                                ))
+                            })?,
+                        ),
+                    }
+                }
             }),
             WellKnownType::TimeDelta(value) => Ok(DuperValue {
                 identifier: Some(
-                    DuperIdentifier::try_from(Cow::Borrowed("TimeDelta"))
-                        .expect("valid identifier"),
+                    DuperIdentifier::try_from(Cow::Borrowed("Duration")).expect("valid identifier"),
                 ),
-                inner: DuperInner::String(DuperString::from(Cow::Owned(value.str()?.extract()?))),
+                inner: {
+                    let py = value.py();
+                    let datetime_module = py.import("datetime")?;
+                    let pydantic_module = py.import("pydantic")?;
+                    let adapter = pydantic_module
+                        .getattr("TypeAdapter")?
+                        .call1((datetime_module.getattr("timedelta")?,))?;
+                    let kwargs = PyDict::new(py);
+                    kwargs.set_item("mode", "json")?;
+                    let dump = adapter
+                        .getattr("dump_python")?
+                        .call((value,), Some(&kwargs))?;
+                    DuperInner::Temporal(
+                        DuperTemporal::try_duration_from(Cow::Owned(dump.extract()?)).map_err(
+                            |err| {
+                                PyValueError::new_err(format!(
+                                    "Failed to parse Temporal value: {err}"
+                                ))
+                            },
+                        )?,
+                    )
+                },
             }),
             WellKnownType::Date(value) => Ok(DuperValue {
                 identifier: Some(
-                    DuperIdentifier::try_from(Cow::Borrowed("Date")).expect("valid identifier"),
+                    DuperIdentifier::try_from(Cow::Borrowed("PlainDate"))
+                        .expect("valid identifier"),
+                ),
+                inner: DuperInner::Temporal(
+                    DuperTemporal::try_plain_date_from(Cow::Owned(
+                        value.call_method0("isoformat")?.extract()?,
+                    ))
+                    .map_err(|err| {
+                        PyValueError::new_err(format!("Failed to parse Temporal value: {err}"))
+                    })?,
                 ),
-                inner: DuperInner::String(DuperString::from(Cow::Owned(
-                    value.call_method0("isoformat")?.extract()?,
-                ))),
             }),
             WellKnownType::Time(value) => Ok(DuperValue {
                 identifier: Some(
-                    DuperIdentifier::try_from(Cow::Borrowed("Time")).expect("valid identifier"),
+                    DuperIdentifier::try_from(Cow::Borrowed("PlainTime"))
+                        .expect("valid identifier"),
+                ),
+                inner: DuperInner::Temporal(
+                    DuperTemporal::try_plain_time_from(Cow::Owned(
+                        value.call_method0("isoformat")?.extract()?,
+                    ))
+                    .map_err(|err| {
+                        PyValueError::new_err(format!("Failed to parse Temporal value: {err}"))
+                    })?,
                 ),
-                inner: DuperInner::String(DuperString::from(Cow::Owned(
-                    value.call_method0("isoformat")?.extract()?,
-                ))),
             }),
             // decimal
             WellKnownType::Decimal(value) => Ok(DuperValue {
@@ -751,6 +868,51 @@ impl<'py> WellKnownType<'py> {
                     value.getattr("pattern")?.extract()?,
                 ))),
             }),
+            // temporal string
+            WellKnownType::TemporalStringInstant(value) => Ok(DuperValue {
+                identifier: Some(DuperIdentifier::try_from("Instant").expect("valid identifier")),
+                inner: DuperInner::Temporal(value),
+            }),
+            WellKnownType::TemporalStringZonedDateTime(value) => Ok(DuperValue {
+                identifier: Some(
+                    DuperIdentifier::try_from("ZonedDateTime").expect("valid identifier"),
+                ),
+                inner: DuperInner::Temporal(value),
+            }),
+            WellKnownType::TemporalStringPlainDate(value) => Ok(DuperValue {
+                identifier: Some(DuperIdentifier::try_from("PlainDate").expect("valid identifier")),
+                inner: DuperInner::Temporal(value),
+            }),
+            WellKnownType::TemporalStringPlainTime(value) => Ok(DuperValue {
+                identifier: Some(DuperIdentifier::try_from("PlainTime").expect("valid identifier")),
+                inner: DuperInner::Temporal(value),
+            }),
+            WellKnownType::TemporalStringPlainDateTime(value) => Ok(DuperValue {
+                identifier: Some(
+                    DuperIdentifier::try_from("PlainDateTime").expect("valid identifier"),
+                ),
+                inner: DuperInner::Temporal(value),
+            }),
+            WellKnownType::TemporalStringPlainYearMonth(value) => Ok(DuperValue {
+                identifier: Some(
+                    DuperIdentifier::try_from("PlainYearMonth").expect("valid identifier"),
+                ),
+                inner: DuperInner::Temporal(value),
+            }),
+            WellKnownType::TemporalStringPlainMonthDay(value) => Ok(DuperValue {
+                identifier: Some(
+                    DuperIdentifier::try_from("PlainMonthDay").expect("valid identifier"),
+                ),
+                inner: DuperInner::Temporal(value),
+            }),
+            WellKnownType::TemporalStringDuration(value) => Ok(DuperValue {
+                identifier: Some(DuperIdentifier::try_from("Duration").expect("valid identifier")),
+                inner: DuperInner::Temporal(value),
+            }),
+            WellKnownType::TemporalString(value) => Ok(DuperValue {
+                identifier: None,
+                inner: DuperInner::Temporal(value),
+            }),
             // uuid
             WellKnownType::Uuid(value) => Ok(DuperValue {
                 identifier: Some(
