@@ -7,8 +7,8 @@ use serde_core::{
 };
 
 use crate::{
-    DuperArray, DuperBytes, DuperIdentifier, DuperInner, DuperKey, DuperObject,
-    DuperObjectTryFromError, DuperString, DuperTuple, DuperValue,
+    DuperArray, DuperBytes, DuperIdentifier, DuperInner, DuperKey, DuperObject, DuperString,
+    DuperTemporal, DuperTuple, DuperValue, serde::error::DuperSerdeError,
 };
 
 impl<'a> DuperValue<'a> {
@@ -41,6 +41,10 @@ impl<'a> DuperValue<'a> {
                 state.serialize_field("inner", &self.inner)?;
                 state.serialize_field("type", "bytes")?;
             }
+            DuperInner::Temporal(temporal) => {
+                state.serialize_field("inner", temporal.as_ref())?;
+                state.serialize_field("type", "temporal")?;
+            }
             DuperInner::Integer(_) => {
                 state.serialize_field("inner", &self.inner)?;
                 state.serialize_field("type", "integer")?;
@@ -70,10 +74,7 @@ impl<'a> DuperValue<'a> {
         'de: 'a,
     {
         let duper_value_meta = DeDuperValue::deserialize(deserializer)?;
-        Ok(DuperValue {
-            identifier: duper_value_meta.identifier,
-            inner: duper_value_meta.inner.try_into().map_err(Error::custom)?,
-        })
+        duper_value_meta.try_into().map_err(Error::custom)
     }
 }
 
@@ -138,20 +139,20 @@ struct DeDuperValue<'b> {
     inner: DeDuperInner<'b>,
 }
 
-impl<'b> TryFrom<DeDuperValue<'b>> for DuperValue<'b> {
-    type Error = DuperObjectTryFromError<'b>;
-
-    fn try_from(value: DeDuperValue<'b>) -> Result<Self, Self::Error> {
-        Ok(DuperValue {
-            identifier: value.identifier,
-            inner: value.inner.try_into()?,
-        })
-    }
-}
-
 struct DeDuperObject<'b>(Vec<(DuperKey<'b>, DeDuperValue<'b>)>);
 struct DeDuperArray<'b>(Vec<DeDuperValue<'b>>);
 struct DeDuperTuple<'b>(Vec<DeDuperValue<'b>>);
+enum DeDuperTemporal<'b> {
+    Instant(Cow<'b, str>),
+    ZonedDateTime(Cow<'b, str>),
+    PlainDate(Cow<'b, str>),
+    PlainTime(Cow<'b, str>),
+    PlainDateTime(Cow<'b, str>),
+    PlainYearMonth(Cow<'b, str>),
+    PlainMonthDay(Cow<'b, str>),
+    Duration(Cow<'b, str>),
+    Unspecified(Cow<'b, str>),
+}
 
 enum DeDuperInner<'b> {
     Object(DeDuperObject<'b>),
@@ -159,6 +160,7 @@ enum DeDuperInner<'b> {
     Tuple(DeDuperTuple<'b>),
     String(DuperString<'b>),
     Bytes(DuperBytes<'b>),
+    Temporal(DeDuperTemporal<'b>),
     Integer(i64),
     Float(f64),
     Boolean(bool),
@@ -173,6 +175,7 @@ impl Display for DeDuperInner<'_> {
             DeDuperInner::Tuple(_) => "tuple",
             DeDuperInner::String(_) => "string",
             DeDuperInner::Bytes(_) => "bytes",
+            DeDuperInner::Temporal(_) => "temporal",
             DeDuperInner::Integer(_) => "integer",
             DeDuperInner::Float(_) => "float",
             DeDuperInner::Boolean(_) => "boolean",
@@ -181,38 +184,90 @@ impl Display for DeDuperInner<'_> {
     }
 }
 
-impl<'b> TryFrom<DeDuperInner<'b>> for DuperInner<'b> {
-    type Error = DuperObjectTryFromError<'b>;
+impl<'b> TryFrom<DeDuperValue<'b>> for DuperValue<'b> {
+    type Error = DuperSerdeError;
 
-    fn try_from(value: DeDuperInner<'b>) -> Result<Self, Self::Error> {
-        match value {
-            DeDuperInner::Object(object) => Ok(DuperInner::Object(DuperObject::try_from(
-                object
-                    .0
-                    .into_iter()
-                    .map(|(key, value)| DuperValue::try_from(value).map(|value| (key, value)))
-                    .collect::<Result<Vec<_>, _>>()?,
-            )?)),
-            DeDuperInner::Array(array) => Ok(DuperInner::Array(DuperArray::from(
-                array
-                    .0
-                    .into_iter()
-                    .map(DuperValue::try_from)
-                    .collect::<Result<Vec<_>, _>>()?,
-            ))),
-            DeDuperInner::Tuple(tuple) => Ok(DuperInner::Tuple(DuperTuple::from(
-                tuple
-                    .0
-                    .into_iter()
-                    .map(DuperValue::try_from)
-                    .collect::<Result<Vec<_>, _>>()?,
-            ))),
-            DeDuperInner::String(string) => Ok(DuperInner::String(string)),
-            DeDuperInner::Bytes(bytes) => Ok(DuperInner::Bytes(bytes)),
-            DeDuperInner::Integer(integet) => Ok(DuperInner::Integer(integet)),
-            DeDuperInner::Float(float) => Ok(DuperInner::Float(float)),
-            DeDuperInner::Boolean(boolean) => Ok(DuperInner::Boolean(boolean)),
-            DeDuperInner::Null => Ok(DuperInner::Null),
+    fn try_from(value: DeDuperValue<'b>) -> Result<Self, Self::Error> {
+        let DeDuperValue { identifier, inner } = value;
+        match inner {
+            DeDuperInner::Object(object) => Ok(DuperValue {
+                identifier,
+                inner: DuperInner::Object(DuperObject::try_from(
+                    object
+                        .0
+                        .into_iter()
+                        .map(|(key, value)| DuperValue::try_from(value).map(|value| (key, value)))
+                        .collect::<Result<Vec<_>, _>>()?,
+                )?),
+            }),
+            DeDuperInner::Array(array) => Ok(DuperValue {
+                identifier,
+                inner: DuperInner::Array(DuperArray::from(
+                    array
+                        .0
+                        .into_iter()
+                        .map(DuperValue::try_from)
+                        .collect::<Result<Vec<_>, _>>()?,
+                )),
+            }),
+            DeDuperInner::Tuple(tuple) => Ok(DuperValue {
+                identifier,
+                inner: DuperInner::Tuple(DuperTuple::from(
+                    tuple
+                        .0
+                        .into_iter()
+                        .map(DuperValue::try_from)
+                        .collect::<Result<Vec<_>, _>>()?,
+                )),
+            }),
+            DeDuperInner::String(string) => Ok(DuperValue {
+                identifier,
+                inner: DuperInner::String(string),
+            }),
+            DeDuperInner::Bytes(bytes) => Ok(DuperValue {
+                identifier,
+                inner: DuperInner::Bytes(bytes),
+            }),
+            DeDuperInner::Temporal(temporal) => Ok(DuperValue {
+                inner: DuperInner::Temporal(match temporal {
+                    DeDuperTemporal::Instant(inner) => DuperTemporal::try_instant_from(inner)?,
+                    DeDuperTemporal::ZonedDateTime(inner) => {
+                        DuperTemporal::try_zoned_date_time_from(inner)?
+                    }
+                    DeDuperTemporal::PlainDate(inner) => DuperTemporal::try_plain_date_from(inner)?,
+                    DeDuperTemporal::PlainTime(inner) => DuperTemporal::try_plain_time_from(inner)?,
+                    DeDuperTemporal::PlainDateTime(inner) => {
+                        DuperTemporal::try_plain_date_time_from(inner)?
+                    }
+                    DeDuperTemporal::PlainYearMonth(inner) => {
+                        DuperTemporal::try_plain_year_month_from(inner)?
+                    }
+                    DeDuperTemporal::PlainMonthDay(inner) => {
+                        DuperTemporal::try_plain_month_day_from(inner)?
+                    }
+                    DeDuperTemporal::Duration(inner) => DuperTemporal::try_duration_from(inner)?,
+                    DeDuperTemporal::Unspecified(inner) => {
+                        DuperTemporal::try_unspecified_from(inner)?
+                    }
+                }),
+                identifier,
+            }),
+            DeDuperInner::Integer(integet) => Ok(DuperValue {
+                identifier,
+                inner: DuperInner::Integer(integet),
+            }),
+            DeDuperInner::Float(float) => Ok(DuperValue {
+                identifier,
+                inner: DuperInner::Float(float),
+            }),
+            DeDuperInner::Boolean(boolean) => Ok(DuperValue {
+                identifier,
+                inner: DuperInner::Boolean(boolean),
+            }),
+            DeDuperInner::Null => Ok(DuperValue {
+                identifier,
+                inner: DuperInner::Null,
+            }),
         }
     }
 }
@@ -429,6 +484,7 @@ enum DeDuperType {
     Tuple,
     String,
     Bytes,
+    Temporal,
     Integer,
     Float,
     Boolean,
@@ -443,6 +499,7 @@ impl Display for DeDuperType {
             DeDuperType::Tuple => "tuple",
             DeDuperType::String => "string",
             DeDuperType::Bytes => "bytes",
+            DeDuperType::Temporal => "temporal",
             DeDuperType::Integer => "integer",
             DeDuperType::Float => "float",
             DeDuperType::Boolean => "boolean",
@@ -461,6 +518,7 @@ impl<'b> TryFrom<&'b str> for DeDuperType {
             "tuple" => Ok(DeDuperType::Tuple),
             "string" => Ok(DeDuperType::String),
             "bytes" => Ok(DeDuperType::Bytes),
+            "temporal" => Ok(DeDuperType::Temporal),
             "integer" => Ok(DeDuperType::Integer),
             "float" => Ok(DeDuperType::Float),
             "boolean" => Ok(DeDuperType::Boolean),
@@ -510,14 +568,11 @@ impl<'de> Visitor<'de> for DeDuperValueVisitor {
                     typ = Some(DeDuperType::try_from(typ_tag).map_err(|_|
                             Error::invalid_value(
                                 serde_core::de::Unexpected::Str(typ_tag),
-                                &"one of: object, array, tuple, string, bytes, integer, float, boolean, null",
+                                &"one of: object, array, tuple, string, bytes, temporal, integer, float, boolean, null",
                             ))?);
                 }
-                _ => {
-                    return Err(Error::unknown_field(
-                        key.as_str(),
-                        &["identifier", "inner", "type"],
-                    ));
+                key => {
+                    return Err(Error::unknown_field(key, &["identifier", "inner", "type"]));
                 }
             }
         }
@@ -532,6 +587,9 @@ impl<'de> Visitor<'de> for DeDuperValueVisitor {
             (DeDuperInner::Tuple(tuple), DeDuperType::Tuple) => DeDuperInner::Tuple(tuple),
             (DeDuperInner::String(string), DeDuperType::String) => DeDuperInner::String(string),
             (DeDuperInner::Bytes(bytes), DeDuperType::Bytes) => DeDuperInner::Bytes(bytes),
+            (DeDuperInner::Temporal(temporal), DeDuperType::Temporal) => {
+                DeDuperInner::Temporal(temporal)
+            }
             (DeDuperInner::Integer(integer), DeDuperType::Integer) => {
                 DeDuperInner::Integer(integer)
             }
@@ -564,6 +622,36 @@ impl<'de> Visitor<'de> for DeDuperValueVisitor {
             {
                 DeDuperInner::String(string)
             }
+            // Temporal from string
+            (DeDuperInner::String(string), DeDuperType::Temporal) => match &identifier {
+                Some(ident) if ident.as_ref() == "Instant" => {
+                    DeDuperInner::Temporal(DeDuperTemporal::Instant(string.into_inner()))
+                }
+                Some(ident) if ident.as_ref() == "ZonedDateTime" => {
+                    DeDuperInner::Temporal(DeDuperTemporal::ZonedDateTime(string.into_inner()))
+                }
+                Some(ident) if ident.as_ref() == "PlainDate" => {
+                    DeDuperInner::Temporal(DeDuperTemporal::PlainDate(string.into_inner()))
+                }
+                Some(ident) if ident.as_ref() == "PlainTime" => {
+                    DeDuperInner::Temporal(DeDuperTemporal::PlainTime(string.into_inner()))
+                }
+                Some(ident) if ident.as_ref() == "PlainDateTime" => {
+                    DeDuperInner::Temporal(DeDuperTemporal::PlainDateTime(string.into_inner()))
+                }
+                Some(ident) if ident.as_ref() == "PlainYearMonth" => {
+                    DeDuperInner::Temporal(DeDuperTemporal::PlainYearMonth(string.into_inner()))
+                }
+                Some(ident) if ident.as_ref() == "PlainMonthDay" => {
+                    DeDuperInner::Temporal(DeDuperTemporal::PlainMonthDay(string.into_inner()))
+                }
+                Some(ident) if ident.as_ref() == "Duration" => {
+                    DeDuperInner::Temporal(DeDuperTemporal::Duration(string.into_inner()))
+                }
+                Some(_) | None => {
+                    DeDuperInner::Temporal(DeDuperTemporal::Unspecified(string.into_inner()))
+                }
+            },
             // Fallback
             (inner, typ) => {
                 return Err(Error::custom(format!(
@@ -597,7 +685,7 @@ mod serde_meta_tests {
 
     use crate::{
         DuperArray, DuperBytes, DuperIdentifier, DuperInner, DuperKey, DuperObject, DuperString,
-        DuperTuple, DuperValue, PrettyPrinter,
+        DuperTemporal, DuperTemporalInner, DuperTuple, DuperValue, PrettyPrinter,
         serde::{de::Deserializer, ser::Serializer},
     };
 
@@ -722,6 +810,15 @@ mod serde_meta_tests {
                         inner: DuperInner::Bytes(DuperBytes::from(&br"/\"[..])),
                     },
                 ),
+                (
+                    DuperKey::from("temporal"),
+                    DuperValue {
+                        identifier: Some(DuperIdentifier(Cow::Borrowed("Instant"))),
+                        inner: DuperInner::Temporal(DuperTemporal::Instant(DuperTemporalInner(
+                            Cow::Borrowed("2022-02-28T03:06:00.092121729Z"),
+                        ))),
+                    },
+                ),
                 (
                     DuperKey::from("integer"),
                     DuperValue {
@@ -773,6 +870,14 @@ mod serde_meta_tests {
                     ),
                     inner: DuperInner::Bytes(DuperBytes::from(&br"/\"[..])),
                 },
+                DuperValue {
+                    identifier: Some(
+                        DuperIdentifier::try_from("MyTemporal").expect("valid identifier"),
+                    ),
+                    inner: DuperInner::Temporal(DuperTemporal::Unspecified(DuperTemporalInner(
+                        Cow::Borrowed("2022-02-28T03:06:00.092121729Z"),
+                    ))),
+                },
                 DuperValue {
                     identifier: Some(DuperIdentifier::try_from("MyInt").expect("valid identifier")),
                     inner: DuperInner::Integer(1337),
