@@ -2,11 +2,12 @@ use std::{borrow::Cow, marker::PhantomData};
 
 use crate::{
     DuperArray, DuperBytes, DuperIdentifier, DuperInner, DuperKey, DuperObject, DuperString,
-    DuperTuple, DuperValue, PrettyPrinter as DuperPrettyPrinter, Serializer as DuperSerializer,
+    DuperTemporal, DuperTuple, DuperValue, PrettyPrinter as DuperPrettyPrinter,
+    Serializer as DuperSerializer,
 };
 use serde_core::{Serialize, ser};
 
-use super::error::Error;
+use super::error::DuperSerdeError;
 
 /// A structure for serializing Rust values into Duper values.
 #[derive(Clone, Default)]
@@ -27,7 +28,7 @@ impl<'a> Serializer<'a> {
 ///
 /// Serialization can fail if `T`'s implementation of [`Serialize`] decides to
 /// fail, or if `T` contains a map with non-string keys.
-pub fn to_duper<'a, T>(value: &'a T) -> Result<DuperValue<'a>, Error>
+pub fn to_duper<'a, T>(value: &'a T) -> Result<DuperValue<'a>, DuperSerdeError>
 where
     T: Serialize,
 {
@@ -41,11 +42,11 @@ where
 ///
 /// Serialization can fail if `T`'s implementation of [`Serialize`] decides to
 /// fail, or if `T` contains a map with non-string keys.
-pub fn to_string<T>(value: &T) -> Result<String, Error>
+pub fn to_string<T>(value: &T) -> Result<String, DuperSerdeError>
 where
     T: Serialize,
 {
-    Ok(DuperSerializer::new(false).serialize(to_duper(value)?))
+    Ok(DuperSerializer::new(false, false).serialize(to_duper(value)?))
 }
 
 /// Serialize the given data structure as a [`String`] of a Duper value, stripping
@@ -55,11 +56,11 @@ where
 ///
 /// Serialization can fail if `T`'s implementation of [`Serialize`] decides to
 /// fail, or if `T` contains a map with non-string keys.
-pub fn to_string_minified<T>(value: &T) -> Result<String, Error>
+pub fn to_string_minified<T>(value: &T) -> Result<String, DuperSerdeError>
 where
     T: Serialize,
 {
-    Ok(DuperSerializer::new(true).serialize(to_duper(value)?))
+    Ok(DuperSerializer::new(true, true).serialize(to_duper(value)?))
 }
 
 /// Serialize the given data structure as a [`String`] of a Duper value,
@@ -69,19 +70,19 @@ where
 ///
 /// Serialization can fail if `T`'s implementation of [`Serialize`] decides to
 /// fail, or if `T` contains a map with non-string keys.
-pub fn to_string_pretty<T>(value: &T, indent: &str) -> Result<String, Error>
+pub fn to_string_pretty<T>(value: &T, indent: &str) -> Result<String, DuperSerdeError>
 where
     T: Serialize,
 {
     Ok(DuperPrettyPrinter::new(false, indent)
-        .map_err(Error::invalid_value)?
+        .map_err(DuperSerdeError::invalid_value)?
         .pretty_print(to_duper(value)?))
 }
 
 impl<'ser, 'a> ser::Serializer for &'ser mut Serializer<'a> {
     type Ok = DuperValue<'a>;
 
-    type Error = Error;
+    type Error = DuperSerdeError;
 
     type SerializeSeq = SerializeSeq<'ser, 'a>;
     type SerializeTuple = SerializeTuple<'ser, 'a>;
@@ -424,7 +425,7 @@ pub struct SerializeSeq<'ser, 'a> {
 
 impl<'ser, 'a> ser::SerializeSeq for SerializeSeq<'ser, 'a> {
     type Ok = DuperValue<'a>;
-    type Error = Error;
+    type Error = DuperSerdeError;
 
     fn serialize_element<T>(&mut self, value: &T) -> Result<(), Self::Error>
     where
@@ -450,7 +451,7 @@ pub struct SerializeTuple<'ser, 'a> {
 
 impl<'ser, 'a> ser::SerializeTuple for SerializeTuple<'ser, 'a> {
     type Ok = DuperValue<'a>;
-    type Error = Error;
+    type Error = DuperSerdeError;
 
     fn serialize_element<T>(&mut self, value: &T) -> Result<(), Self::Error>
     where
@@ -478,7 +479,7 @@ pub struct SerializeTupleStruct<'ser, 'a> {
 // Serialize struct Rgb(u8, u8, u8) as Rgb((..., ..., ...))
 impl<'ser, 'a> ser::SerializeTupleStruct for SerializeTupleStruct<'ser, 'a> {
     type Ok = DuperValue<'a>;
-    type Error = Error;
+    type Error = DuperSerdeError;
 
     fn serialize_field<T>(&mut self, value: &T) -> Result<(), Self::Error>
     where
@@ -508,7 +509,7 @@ pub struct SerializeTupleVariant<'ser, 'b> {
 // Serialize enum E { T(u8, u8) } as E({T: (..., ...)})
 impl<'ser, 'b> ser::SerializeTupleVariant for SerializeTupleVariant<'ser, 'b> {
     type Ok = DuperValue<'b>;
-    type Error = Error;
+    type Error = DuperSerdeError;
 
     fn serialize_field<T>(&mut self, value: &T) -> Result<(), Self::Error>
     where
@@ -546,7 +547,7 @@ pub struct SerializeMap<'ser, 'a> {
 
 impl<'ser, 'a> ser::SerializeMap for SerializeMap<'ser, 'a> {
     type Ok = DuperValue<'a>;
-    type Error = Error;
+    type Error = DuperSerdeError;
 
     fn serialize_key<T>(&mut self, key: &T) -> Result<(), Self::Error>
     where
@@ -562,7 +563,7 @@ impl<'ser, 'a> ser::SerializeMap for SerializeMap<'ser, 'a> {
                 self.next_key = Some(DuperKey::from(s.into_inner()));
                 Ok(())
             }
-            _ => Err(Error::serialization("map key must be a string")),
+            _ => Err(DuperSerdeError::serialization("map key must be a string")),
         }
     }
 
@@ -575,7 +576,7 @@ impl<'ser, 'a> ser::SerializeMap for SerializeMap<'ser, 'a> {
             self.entries.push((key, value));
             Ok(())
         } else {
-            Err(Error::serialization(
+            Err(DuperSerdeError::serialization(
                 "serialize_value called before serialize_key",
             ))
         }
@@ -598,7 +599,7 @@ pub struct SerializeStruct<'ser, 'a> {
 // Serialize struct Rgb { r: u8, g: u8, b: u8 } as Rgb({r: ..., g: ..., b: ...})
 impl<'ser, 'a> ser::SerializeStruct for SerializeStruct<'ser, 'a> {
     type Ok = DuperValue<'a>;
-    type Error = Error;
+    type Error = DuperSerdeError;
 
     fn serialize_field<T>(&mut self, key: &'static str, value: &T) -> Result<(), Self::Error>
     where
@@ -611,11 +612,176 @@ impl<'ser, 'a> ser::SerializeStruct for SerializeStruct<'ser, 'a> {
     }
 
     fn end(self) -> Result<Self::Ok, Self::Error> {
-        Ok(DuperValue {
-            identifier: (!self.name.is_empty())
-                .then_some(DuperIdentifier::try_from_lossy(Cow::Borrowed(self.name))?),
-            inner: DuperInner::Object(DuperObject::try_from(self.fields)?),
-        })
+        // Special handling for Temporal values
+        if self.name == super::temporal::STRUCT {
+            let mut typ: Option<DuperValue<'a>> = None;
+            let mut value: Option<DuperValue<'a>> = None;
+            for (key, val) in self.fields.into_iter() {
+                match key.as_ref() {
+                    super::temporal::FIELD_TYPE => typ = Some(val),
+                    super::temporal::FIELD_VALUE => value = Some(val),
+                    field => {
+                        return Err(DuperSerdeError::invalid_value(format!(
+                            "unknown field {field} for TemporalString",
+                        )));
+                    }
+                }
+            }
+
+            let typ = typ.ok_or_else(|| {
+                DuperSerdeError::invalid_value(format!(
+                    "missing field {} for TemporalString",
+                    super::temporal::FIELD_TYPE
+                ))
+            })?;
+            let value = value.ok_or_else(|| {
+                DuperSerdeError::invalid_value(format!(
+                    "missing field {} for TemporalString",
+                    super::temporal::FIELD_VALUE
+                ))
+            })?;
+
+            match (typ.inner, value.inner) {
+                (DuperInner::String(typ), DuperInner::String(value)) => match typ.as_ref() {
+                    "Instant" => Ok(DuperValue {
+                        identifier: Some(
+                            DuperIdentifier::try_from("Instant").expect("valid identifier"),
+                        ),
+                        inner: DuperInner::Temporal(
+                            DuperTemporal::try_instant_from(value.into_inner()).map_err(|err| {
+                                DuperSerdeError::invalid_value(format!(
+                                    "failed to parse Instant for TemporalString: {err}"
+                                ))
+                            })?,
+                        ),
+                    }),
+                    "ZonedDateTime" => Ok(DuperValue {
+                        identifier: Some(
+                            DuperIdentifier::try_from("ZonedDateTime").expect("valid identifier"),
+                        ),
+                        inner: DuperInner::Temporal(
+                            DuperTemporal::try_zoned_date_time_from(value.into_inner()).map_err(
+                                |err| {
+                                    DuperSerdeError::invalid_value(format!(
+                                        "failed to parse ZonedDateTime for TemporalString: {err}"
+                                    ))
+                                },
+                            )?,
+                        ),
+                    }),
+                    "PlainDate" => Ok(DuperValue {
+                        identifier: Some(
+                            DuperIdentifier::try_from("PlainDate").expect("valid identifier"),
+                        ),
+                        inner: DuperInner::Temporal(
+                            DuperTemporal::try_plain_date_from(value.into_inner()).map_err(
+                                |err| {
+                                    DuperSerdeError::invalid_value(format!(
+                                        "failed to parse PlainDate for TemporalString: {err}"
+                                    ))
+                                },
+                            )?,
+                        ),
+                    }),
+                    "PlainTime" => Ok(DuperValue {
+                        identifier: Some(
+                            DuperIdentifier::try_from("PlainTime").expect("valid identifier"),
+                        ),
+                        inner: DuperInner::Temporal(
+                            DuperTemporal::try_plain_time_from(value.into_inner()).map_err(
+                                |err| {
+                                    DuperSerdeError::invalid_value(format!(
+                                        "failed to parse PlainTime for TemporalString: {err}"
+                                    ))
+                                },
+                            )?,
+                        ),
+                    }),
+                    "PlainDateTime" => Ok(DuperValue {
+                        identifier: Some(
+                            DuperIdentifier::try_from("PlainDateTime").expect("valid identifier"),
+                        ),
+                        inner: DuperInner::Temporal(
+                            DuperTemporal::try_plain_date_time_from(value.into_inner()).map_err(
+                                |err| {
+                                    DuperSerdeError::invalid_value(format!(
+                                        "failed to parse PlainDateTime for TemporalString: {err}"
+                                    ))
+                                },
+                            )?,
+                        ),
+                    }),
+                    "PlainYearMonth" => Ok(DuperValue {
+                        identifier: Some(
+                            DuperIdentifier::try_from("PlainYearMonth").expect("valid identifier"),
+                        ),
+                        inner: DuperInner::Temporal(
+                            DuperTemporal::try_plain_year_month_from(value.into_inner()).map_err(
+                                |err| {
+                                    DuperSerdeError::invalid_value(format!(
+                                        "failed to parse PlainYearMonth for TemporalString: {err}"
+                                    ))
+                                },
+                            )?,
+                        ),
+                    }),
+                    "PlainMonthDay" => Ok(DuperValue {
+                        identifier: Some(
+                            DuperIdentifier::try_from("PlainMonthDay").expect("valid identifier"),
+                        ),
+                        inner: DuperInner::Temporal(
+                            DuperTemporal::try_plain_month_day_from(value.into_inner()).map_err(
+                                |err| {
+                                    DuperSerdeError::invalid_value(format!(
+                                        "failed to parse PlainMonthDay for TemporalString: {err}"
+                                    ))
+                                },
+                            )?,
+                        ),
+                    }),
+                    "Duration" => Ok(DuperValue {
+                        identifier: Some(
+                            DuperIdentifier::try_from("Duration").expect("valid identifier"),
+                        ),
+                        inner: DuperInner::Temporal(
+                            DuperTemporal::try_duration_from(value.into_inner()).map_err(
+                                |err| {
+                                    DuperSerdeError::invalid_value(format!(
+                                        "failed to parse Duration for TemporalString: {err}"
+                                    ))
+                                },
+                            )?,
+                        ),
+                    }),
+                    "Unspecified" => Ok(DuperValue {
+                        identifier: Some(
+                            DuperIdentifier::try_from("Unspecified").expect("valid identifier"),
+                        ),
+                        inner: DuperInner::Temporal(
+                            DuperTemporal::try_unspecified_from(value.into_inner()).map_err(
+                                |err| {
+                                    DuperSerdeError::invalid_value(format!(
+                                        "failed to parse Unspecified for TemporalString: {err}"
+                                    ))
+                                },
+                            )?,
+                        ),
+                    }),
+                    _ => Err(DuperSerdeError::invalid_value(format!(
+                        "invalid type {typ:?} for TemporalString",
+                    ))),
+                },
+                _ => Err(DuperSerdeError::invalid_value(
+                    "invalid fields for TemporalString",
+                )),
+            }
+        } else {
+            Ok(DuperValue {
+                identifier: (!self.name.is_empty())
+                    .then_some(DuperIdentifier::try_from_lossy(Cow::Borrowed(self.name))?),
+                inner: DuperInner::Object(DuperObject::try_from(self.fields)?),
+            })
+        }
     }
 }
 
@@ -629,7 +795,7 @@ pub struct SerializeStructVariant<'ser, 'a> {
 // Serialize enum E { S { x: i32, y: String } } as E({S: {x: ..., y: ...}})
 impl<'ser, 'a> ser::SerializeStructVariant for SerializeStructVariant<'ser, 'a> {
     type Ok = DuperValue<'a>;
-    type Error = Error;
+    type Error = DuperSerdeError;
 
     fn serialize_field<T>(&mut self, key: &'static str, value: &T) -> Result<(), Self::Error>
     where
